'''
fix_imap_internaldate.py - Fix the INTERNALDATE field on IMAP servers

Copyright (c) 2012 Intra2net AG
Author: Plamen Dimitrov

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
'''

import sys
import csv
import logging, configparser
from date_interpreter import DateInterpreter
from mail_iterator import MailIterator
from caching_data import CachingData

def main():
    """Iterates through csv list of users and their mailboxes"""
    if(len(sys.argv) > 1):
        if(sys.argv[1]=="--h"):
            print("The default mode of the script is test mode."
                  "Add '--u' argument to exit to modify messages."
                  "For a detailed list of each message with a date conflict change"
                  "change the 'log_level' in the configuration file from '30' to '20'.")
            return
        if(sys.argv[1]=="--u"):
            test_mode = False
    else:
        test_mode = True

    config = load_configuration()
    logging.basicConfig(filename='fix_imap_internaldate.log',
                        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                        level=config.getint('basic_settings', 'log_level'))

    date_interp = DateInterpreter()
    caching_data = CachingData()
    logging.warning("Cache version %s loaded.\n\n", caching_data.version)
    user_reader = csv.DictReader(open("userdata.csv", "r"), delimiter=',')

    server = config.get('basic_settings', 'imap_server')
    tolerance = config.getint('basic_settings', 'tolerance')

    for user in user_reader:
        try:
            session = MailIterator(server, user['username'], user['password'])
        except UserWarning as ex:
            logging.error(ex)
            continue
        for mailbox in session:
            try:
                box = caching_data.retrieve_cached_mailbox(mailbox[0], mailbox[1], user['username'])
                mail_ids = session.fetch_messages()
                new_ids = box.synchronize(mail_ids)
                logging.warning("%s non-cached messages found out of %s in %s.\n", len(new_ids), len(mail_ids), box.name)
            except UserWarning as ex:
                logging.error(ex)
                continue
            for mid in new_ids:
                try:
                    fetched_internal_date = session.fetch_internal_date(mid)
                    internal_date = date_interp.extract_internal_date(fetched_internal_date)
                    fetched_received_date = session.fetch_received_date(mid)
                    received_date = date_interp.extract_received_date(fetched_received_date)
                    if(received_date==""):
                        logging.info("No received date could be found in message uid: %s - mailbox: %s - user: %s.\n",
                                        mid.decode("utf-8"), box.name, box.owner)
                        box.no_received_field += 1
                        continue
                except UserWarning as ex:
                    logging.error(ex)
                    continue
                if(date_interp.compare_dates(received_date, internal_date, tolerance)):
                    #print(received_date, internal_date)
                    if(test_mode==0):
                        try:
                            session.update_message(mid, box.name, received_date)
                        except UserWarning as ex:
                            logging.error(ex)
                            continue
                    else:
                        logging.info("Date conflict found in message uid: %s - mailbox: %s - user: %s.\nInternal date %s is different from received date %s from RECEIVED header:\n%s.",
                                        mid.decode("utf-8"), box.name, box.owner,
                                        internal_date.strftime("%d %b %Y %H:%M:%S"),
                                        received_date.strftime("%d %b %Y %H:%M:%S"),
                                        fetched_received_date[0][1].decode("utf-8").split("Received:")[1])
                    # count total emails for every user and mailbox
                    box.date_conflicts += 1
            # if all messages were successfully fixed confirm caching
            if(not test_mode):
                box.confirm_change()
        
        # final report on date conflicts
        caching_data.report_date_conflicts()

def load_configuration():
    """Loads the script configuration from a file or creates such."""
    config = configparser.RawConfigParser()    
    try:
        config.read('confscript.cfg')
    except IOError:
        config.add_section('basic_settings')
        config.set('basic_settings', 'log_level', logging.DEBUG)
        #config.set('Basic settings', 'bool', 'true')
        with open('confscript.cfg', 'w') as configfile:
            config.write(configfile)
    return config

if(__name__ == "__main__"):
    main()
